@@ -1,5 +1,4 @@
-from meloetta.frameworks.porygon.model.encoders.private_encoder import *
-from meloetta.frameworks.porygon.model.encoders.public_encoder import *
+from meloetta.frameworks.porygon.model.encoders.side_encoder import *
 from meloetta.frameworks.porygon.model.encoders.weather_encoder import (
     WeatherEncoder,
 )
